<?php

chdir(dirname(__FILE__));
require_once '../gui/db_utils.inc.php';
require_once '/opt/monitor/op5/reports/gui/auth.php';
require_once '/opt/monitor/op5/reports/gui/lib_gui.php';

define("DRY_RUN", false);


$avail_config_cols =
	'user,'.
	'report_name,'.
	'created,'.
	'rpttimeperiod,'.
	'report_period,'.
#	'start_time,'.
#	'end_time,'.
	'report_type,'.
	'initialassumedhoststate,'.
	'initialassumedservicestate,'.
	'assumeinitialstates,'.
	'scheduleddowntimeasuptime,'.
	'assumestatesduringnotrunning,'.
	'includesoftstates,'.
	'updated';

$avail_config_object_cols = "avail_id, name";

$auto_reports_scheduled_cols =
	'user, '.
	'report_type_id, '.
	'report_id, '.
	'recipients, '.
	'period_id';
/*

steps:

1  SELECT * FROM auto_reports
2a INSERT INTO avail_config
2b INSERT INTO avail_config_objects
2c INSERT INTO auto_reports_scheduled

*/

if(DRY_RUN)
{
	print "PERFORMING DRY RUN, DATABASE WILL REMAIN UNCHANGED\n";
}
gui_db_connect();

# 1) lookup periods
$periods = array();
$q = "SELECT * FROM auto_reports_periods";
$res = sql_exec_query($q);
while($row = sql_fetch_array($res))
{
	$periods[strtolower($row['periodname'])] = $row['id'];
}

# 2) lookup avail report type
$q = "SELECT id FROM auto_report_types WHERE identifier='avail'";
$res = sql_exec_query($q);
$row = sql_fetch_row($res);
$report_type_id = $row[0];

$q = "SELECT * FROM auto_reports";
$res = sql_exec_query($q);

$num_reports = 0;
while($row = sql_fetch_array($res))
{
	$num_reports++;
	if(DRY_RUN)
		print "Converting report $num_reports\n";

	$host 							= $row['host'];
	$service 						= $row['service'];
	$hostgrp 						= $row['hostgrp'];
	$servicegrp 					= $row['servicegrp'];

	$report_type 					= !empty($service) ? 'services' : (!empty($host) ? 'hosts' : (!empty($hostgrp) ? 'hostgroups' : 'servicegroups'));
	if(strpos('host', $report_type) !== false)
		$subtype = 'host';
	else
		$subtype = 'service';

	$initialassumedhoststate 		= convert_assumed_state($row['initialassumedhoststate'], $subtype);
	$initialassumedservicestate 	= convert_assumed_state($row['initialassumedservicestate'], $subtype);

	$period_id 						= $periods[strtolower($row['reporttime'])]; # weekly/monthly
	$timeperiod 					= $row['timeperiod'];
	$rpttimeperiod 					= $row['rpttimeperiod'];
	$contacts 						= $row['contacts'];
	$backtrack 						= $row['backtrack'];
	$user 							= $row['user'];

	$includesoftstates 				= convert_yesno_int($row['includesoftstates']);
	$assumestatesduringnotrunning 	= convert_yesno_int($row['assumestatesduringnotrunning']);
	$assumestateretention 			= convert_yesno_int($row['assumestateretention']);
	$assumeinitialstates 			= convert_yesno_int($row['assumeinitialstates']);
	$scheduleddowntimeasuptime 		= convert_yesno_int($row['scheduleddowntimeasuptime']);

	$report_name = "converted availability report ".date("Y-m-d")." old id=".$row['id'];


	# 2a) insert into avail_config

	$avail_config_vals = "('$user', '$report_name', NOW(), '$rpttimeperiod', '$timeperiod', '$report_type', '$initialassumedhoststate', ".
						 "'$initialassumedservicestate', '$assumeinitialstates', '$scheduleddowntimeasuptime', '$assumestatesduringnotrunning', '$includesoftstates', NOW())";

	$q = "INSERT INTO monitor_reports.avail_config ($avail_config_cols) VALUES $avail_config_vals";
	if(DRY_RUN)
		print "$q\n";
	else
		sql_exec_query($q);

	switch($report_type)
	{
		case 'hosts':		$object	= $host; break;
		case 'services': 	$object = ($service=='all' ? 'all' : "$host;$service"); break;
		case 'hostgroups':	$object	= $hostgrp; break;
		default:			$object	= $servicegrp;
	}

	$report_id = mysql_insert_id();


	# 2b) insert into avail_config_objects

	# instantiate 'all', since new reports currently does not support this

	if($object == 'all')
	{
		$mon_auth = new monitor_auth_class($user);

		switch($report_type)
		{
			case 'hosts': 		$objs = $mon_auth->get_authorized_hosts(); break;
			case 'services': 	$objs = $mon_auth->get_authorized_services(); break;
			case 'hostgroups': 	$objs = $mon_auth->get_authorized_hostgroups(); break;
			default: 			$objs = $mon_auth->get_authorized_servicegroups();
		}

		if(count($objs) > 0)
		{
			$avail_config_object_vals = "($report_id, '".implode("'), ($report_id, '", array_values($objs))."')";
		}
		else
		{
			$avail_config_object_vals = '';
		}
	}
	else
	{
		$avail_config_object_vals = "($report_id, '$object')";
	}

	$q = "INSERT INTO monitor_reports.avail_config_objects ($avail_config_object_cols) VALUES $avail_config_object_vals";

	if($avail_config_object_vals != '')
	{
		if(DRY_RUN)
			print "$q\n";
		else
			sql_exec_query($q);
	}
	else
		error_log("converting avail report id(#$report_id) without any objects!");


	# 2c) insert into auto_reports_scheduled

	$auto_reports_scheduled_vals = "('$user', $report_type_id, $report_id, '$contacts', $period_id)";
	$q = "INSERT INTO auto_reports_scheduled ($auto_reports_scheduled_cols) VALUES $auto_reports_scheduled_vals";
	if(DRY_RUN)
		print "$q\n";
	else
		sql_exec_query($q);


	$q = "UPDATE auto_reports SET avail_config_id = $report_id WHERE id = ".$row['id'];
	sql_exec_query($q);
}

error_log("conversion of $num_reports scheduled reports completed");
if(DRY_RUN)
	print "Dry run completed\n";

?>
