<?php
include_once 'config.inc.php';
include_once 'system.inc.php';
include_once 'data.inc.php';

/* makes a $sep-separated string of $arr's association keys OR it's
 * values, depending on whether ALL values are 'ON' or something else */
function stringify_array ($arr) {
	$str = '';

	/* handle 'n' (for never) option for [host|service]_notification_options */
	if (isset ($arr['n'])) {
		if (count ($arr) == 1) return ('n');
		else unset ($arr['n']);
	}

	foreach ($arr as $k => $v) {
		if ($v == 'on') {
			$str = implode (',', array_keys ($arr));
		}
		else $str = implode (',', $arr);
		break;
	}

	return ($str);
}

/****************************************************************************/
/************************ PRETTY-PRINT FUNCTIONS ****************************/
/****************************************************************************/

/* Makes a selection of the items in the indexed array $lines, using $dflt
 * as default selection, with a (form) 'name' of $select_name */
function make_selection ($tokenvalues, $dflt, $select_name = 'selection') {
	echo html_font();
	/* if array is empty, we tell the user so */
	if(!is_array($tokenvalues)) {
		echo "None configured\n";
		return(true);
	}

	if(count($tokenvalues) === 1) foreach($tokenvalues as $line) {
		echo "<input type='hidden' name='$select_name' value='$line' />\n";
		echo "$line (only one configured)\n";
		return(true);
	}

	/* set up the selection */
	echo "\n<select name='$select_name'>\n";

	/* iterate the values */
	foreach ($tokenvalues as $line) {
		if ($line === $dflt)
		  echo "  <option selected=\"selected\" value=\"$line\">$line</option>\n";
		else
		  echo "  <option value=\"$line\">$line</option>\n";
	}
	echo "</select>\n";

	return true;
}

/* makes a multiple choice selection pulldown */
function make_multi_selection ($tokenvalues, $dflt, $select_name = 'selection') {
	echo html_font();
	/* if array is empty or only has one object, we tell the user so */
	if(!is_array($tokenvalues)) {
		echo "None configured (multiple selection possible)\n";
		return true;
	}

	if(count($tokenvalues) === 1) foreach($tokenvalue as $line) {
		echo "<input type='hidden' name='$select_name' value='$line' />\n";
		echo "$line (only one configured)\n";
		break;
	}

	/* set up the selection */
	echo "\n<select name='{$select_name}[]' multiple=\"multiple\">\n";

	/* iterate the values */
	$dflt = split ("[\t ]*,[\t ]*", $dflt);
	foreach ($tokenvalues as $line) {
		if ($dflt && my_array_search ($line, $dflt))
		  echo "  <option selected=\"selected\" value=\"$line\">$line</option>\n";
		else
		  echo "  <option value=\"$line\">$line</option>\n";
	}
	echo "</select>\n";

	return true;
}

/*
 * makes a series of checkboxes in a vertical table.
 * $options is an associative array containing all the possible options,
 *          with the values being the output for each option.
 * $input is an indexed array containing the set values
 * $name is the form name we give it. Suffixed by '[]' */
function make_checkboxes ($options, $obj_val, $name) {
	$cur = split (',', $obj_val);
	echo '<table class="max" summary="">'."\n";
	foreach ($options as $opt => $opt_str) {
		if ($opt_str == 'noshow') {
			echo '<input type="hidden" name="'.$name.'['.$opt.']" value="'.$opt_str.'" />'."\n";
			continue;
		}

		echo "<tr>\n";
		printf ("  <td><input type='checkbox' name='%s[%s]' %s>\n",
				$name, $opt,
				my_array_search ($opt, $cur) ? "checked='true'" : "");
		echo "  </td><td></td>\n";
		echo "  <td>" . html_font() . "$opt_str</font></td>\n</tr>";
	}
	echo "</table>\n";
}

/****************************************************************************/
/**************************** MISC FUNCTIONS ********************************/
/****************************************************************************/
/*
 * 'The other ones'
 */

/* should be replaced with something fancier later */
function is_super_user () {
	global $UserAdmin;
	global $Config;

//	return(true);
	$user = $_SERVER['REMOTE_USER'];
	if($user === 'exon' || $user === 'ae') return(true);
	if ($user === $UserAdmin) return (true);
	$buf = read_file ($Config['cgi']);
	$lines = explode("\n", $buf);
	foreach($lines as $line) {
		$line = trim($line);
		if(!strlen($line)) continue;
		if($line[0] == '#') continue;
		if(strstr($line, 'authorized_for_system_commands')) {
			$part = split(" *= *", $line, 2);
			$users = split(" *, *", $part[1]);
			foreach($users as $u) {
				if($u == "*") return(true);
				if($u == $user) return(true);
			}
		}
	}

	return (false);
}

/* validates a user. Exits with an errormessage if not auth'ed ok. */
function validate_user() {
	global $Config;
	/* set up the playground. 'tedious' tasks end up here. */
	if (isset ($_REQUEST['action'])) {
	  $_REQUEST['action'] = trim ($_REQUEST['action']);
	if($_REQUEST['action'] === 'Apply Changes')
	  $_REQUEST['action'] = 'Apply';
	}
	/* superuser(s) can do anything, and only they can add reports */
	if (is_super_user()) return (true);

	/* Unauthorized ... */
	html_start ();
	echo "<p><strong>Access denied</strong><br />\n";
	echo "It appears as though you aren't authorized to access the\n";
	echo "configuration interface.<br /><br />\n";
	echo "Note that you must be listed as\n";
	echo "'<strong>authorized_for_configuration_information</strong>' in the file\n";
	echo "'<strong>cgi.cfg</strong>', which is part of OP5 Monitors configuration.</p>\n";
	html_end ();
	exit ();
}

/* sets a specified font */
function html_font() {
	global $Font_Face;

	$font_colors = array('white' => 'ffffff');

	$size = 2;
	$color = '#000000';

	$numargs = func_num_args();
	if($numargs) $size = func_get_arg(0);
	if($numargs == 2) $color = "#" . $font_colors[func_get_arg(1)];

	//return "<font face='" . $Font_Face . "' size='$size' color='$color'>";
	return '';
}

/* sets up the playground. All pages get the same generic look. */
function html_start () {
	global $Version;
	$user = $_SERVER['REMOTE_USER'];
	$date_str = date('D M h:i:s T Y');

	header('Cache-Control: no-cache');
	echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'."\n\n";
	echo '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">'."\n\t";
	echo '<head>'."\n\t\t";
	echo '<meta http-equiv="content-type" content="application/xhtml+xml; charset=iso-8859-1" />'."\n\t\t";
	echo '<meta name="robots" content="noindex, nofollow" />'."\n\t\t";
	echo '<meta name="author" content="My Appelgren, op5 AB" />'."\n\t\t";
	echo '<title> op5 Monitor &raquo; Schedule Reports</title>'."\n\t\t";
	echo '<link type="text/css" rel="stylesheet" href="css/default.css" />'."\n\t";
	echo '</head>'."\n\n\t";

	echo "<body>\n\t\t";
	echo '<div id="menubox">'.
			  "<ul><li><a href=\"/monitor/op5/auto-reports/auto-reports.php\">Schedule new report</a></li>\n" .
			  "<li><a href=\"/monitor/op5/auto-reports/scheduled.php\">View Scheduled reports</a></li></ul>\n".
			'</div>'."\n";

}

function html_end () {
	//echo '<div id="footer">Copyright &copy; 2003-'.date('Y').' op5 AB. All rights reserved</div>'."\n";
	echo "\t</body>\n</html>";
	exit (0);
}

/* searches an array properly (array_search seems broken) */
function my_array_search ($value, $arr) {
	if(!is_array($arr)) return(false);
	foreach ($arr as $aval) if ($value == $aval) return true;
	return false;
}

/**
 * Fetch information from database on what sorts of
 * reports this system are able to produce
 *
 */
function get_available_report_types()
{
	gui_db_connect();
	$sql = "SELECT * FROM auto_report_types ORDER BY name";
	$res = sql_exec_query($sql);
	gui_db_close();
	return $res;
}

function get_available_report_periods()
{
	$sql = "SELECT * from auto_reports_periods";
	gui_db_connect();
	$res = sql_exec_query($sql);
	gui_db_close();
	return $res;
}

function fetch_report_path($type_id=false, $is_ajax=true)
{
	if ($is_ajax) {
		// Instantiate the xajaxResponse object
		$objResponse = new xajaxResponse();
		$objResponse->call("show_progress", "progress", 'Please wait...');
	}
	if (!$type_id) return false;

	// fetch info on selected id
	gui_db_connect();
	if (is_array($type_id)) $type_id = $type_id[0];
	$sql = "SELECT * FROM auto_report_types WHERE id=".(int)$type_id;
	$res = sql_exec_query($sql);

	if (!$res) {
		if ($is_ajax) {
			$objResponse->assign("saved_reports_holder","innerHTML", "FAILED fetching path");
		} else {
			return false;
		}
	}
	$info = sql_fetch_array($res);
	$path = $info['script_reports_path'];
	gui_db_close();

	if ($is_ajax) {
		$objResponse->call('show_reports', $path);
		$objResponse->call('hide_progress');
		return $objResponse;
	} else {
		return $path;
	}
}

function fetch_module_reports($type_id=false,$is_ajax=true)
{
	if ($is_ajax) {
		$objResponse = new xajaxResponse();
		$objResponse->call("show_progress", "progress", 'Please wait...');
	}
	if (is_array($type_id)) $type_id = $type_id[0];
	$type_id = (int)$type_id;
	// fetch info on selected id
	gui_db_connect();
	if (!$type_id) {
		$sql = "SELECT * FROM auto_report_types";
	} else {
		$sql = "SELECT * FROM auto_report_types WHERE id=".$type_id;
	}
	$res = sql_exec_query($sql);

	if (!$res || mysql_num_rows($res)==0 || mysql_errno()) {
		if ($is_ajax) {
			$objResponse->assign("err_msg","innerHTML", "FAILED fetching path");
		} else {
			return false;
		}
	}

	// if we got no type and we aren't in the middle of an ajax call
	// we are only interested in the actual result for further
	// processing by PHP
	if (!$type_id && !$is_ajax) {
		return $res;
	}

	$info = sql_fetch_array($res);
	$path = $info['script_reports_path'];
	gui_db_close();

	// fetch data from module by including the file
	// with path stored in db
	$return = false;
	include($path);
	if (!empty($return)) {
		if ($is_ajax) {
			$objResponse->call('show_reports', convert_json($return));
		} else {
			return $return;
		}
	} else {
		if ($is_ajax) {
			$objResponse->assign("err_msg","innerHTML", "Found no saved reports.<br />Please create and save a report using the links in the menu on the left.");
			$objResponse->call("hide_rows");
		} else {
			return false;
		}
	}
	$objResponse->call('hide_progress');
	return $objResponse;
}

function edit_report($id=false, $rep_type=false, $saved_report_id=false, $period=false, $recipients=false, $filename='', $description='')
{
	gui_db_connect();
	$id 			= (int)$id;
	$rep_type 		= (int)$rep_type;
	$saved_report_id 	= (int)$saved_report_id;
	$period			= (int)$period;
	$recipients 		= trim(mysql_real_escape_string($recipients));
	$filename		= trim(mysql_real_escape_string($filename));
	$description	= trim(mysql_real_escape_string($description));
	if (isset($_SERVER['REMOTE_USER'])) {
		$user = $_SERVER['REMOTE_USER'];
	} else {
		return "Unable to save scheduled report since it is impossible to detect current user.\nPlease try again.";
	}

	if (!$rep_type || !$saved_report_id || !$period || empty($recipients)) return false;

	// some users might use ';' to separate email adresses
	// just replace it with ',' and continue
	$recipients = str_replace(';', ',', $recipients);
	$rec_arr = explode(',', $recipients);
	if (!empty($rec_arr)) {
		foreach ($rec_arr as $recipient) {
			if (trim($recipient)!='') {
				$checked_recipients[] = trim($recipient);
			}
		}
		$recipients = implode(', ', $checked_recipients);
	}

	if ($id) {
		// UPDATE
		$sql = "UPDATE auto_reports_scheduled SET user='".$user."', report_type_id=".$rep_type.", report_id=".$saved_report_id.",
			recipients='".$recipients."', period_id=".$period.", filename='".$filename."', description='".$description."' WHERE id=".$id;
	} else {
		$sql = "INSERT INTO auto_reports_scheduled (user, report_type_id, report_id, recipients, period_id, filename, description)
			VALUES('".$user."', ".$rep_type.", ".$saved_report_id.", '".$recipients."', ".$period.", '".$filename."', '".$description."');";
	}

	$res = sql_exec_query($sql);
	if (!$res || mysql_errno()) {
		return mysql_error();
	}
	if (!$id) {
		$id = mysql_insert_id();
	}
	gui_db_close();
	return $id;
}

function delete_schedule($id=false)
{
	$id = (int)$id;
	$objResponse = new xajaxResponse();
	$objResponse->call("show_progress", "progress", 'Please wait');
	if (!$id) {
		#$objResponse->call("show_progress", "progress", 'Please wait');
		$objResponse->assign("err_msg","innerHTML", "Missing ID so nothing to delete");
		return $objResponse;
	}
	$sql = "DELETE FROM auto_reports_scheduled WHERE id=".$id;
	gui_db_connect();
	$res = sql_exec_query($sql);
	gui_db_close();
	if (!$res || mysql_errno()) {
		return mysql_error();
		$objResponse->assign("err_msg","innerHTML", "Error when trying to delete scheduled report with ID $id: " . mysql_error());
		return $objResponse;
	}
	$objResponse->call('hide_progress');
	#$objResponse->assign("err_msg","innerHTML", "DELETED");
	$objResponse->call('remove_deleted_row', $id);
	return $objResponse;
}

function send_report_now($id)
{
	$id = (int)$id;
	$objResponse = new xajaxResponse();

	gui_db_connect();

	$sql = "SELECT * FROM auto_reports_scheduled JOIN auto_report_types ON report_type_id=auto_report_types.id WHERE auto_reports_scheduled.id=$id";
	$res = sql_exec_query($sql);

	$row = sql_fetch_array($res);
	mysql_free_result($res);

	if($row)
	{
		if(empty($row['recipients']))
		{
			$objResponse->assign("err_msg","innerHTML", "No recipients for selected report");
			$objResponse->call('hide_progress');
			return $objResponse;
		}

		$report_id 		= $row['report_id'];
		$report_type_id = $row['report_type_id'];
		$recipients 	= str_replace(' ', '', $row['recipients']);
		$filename 		= $row['filename'];
		$runner_script  = $row['script_reports_run'];

		$argc = 4;
		$argv = array($runner_script, $report_id, $recipients, $filename);

		include $runner_script;
		$objResponse->call("send_status",1);
	}
	else
	{
		$objResponse->call("send_status",0);
	}

	gui_db_close();

	$objResponse->call('hide_progress', 3000); # hide message after 3 seconds

	return $objResponse;
}

function fetch_saved_report_types()
{
	if (isset($_SERVER['REMOTE_USER'])) {
		$user = $_SERVER['REMOTE_USER'];
	} else {
		return false;
	}

	$sql = "SELECT DISTINCT(ars.report_type_id), art.name FROM auto_reports_scheduled ars, auto_report_types art
		WHERE ars.user='".$user."' AND art.id=ars.report_type_id ORDER BY art.name";
	gui_db_connect();
	$res = sql_exec_query($sql);
	gui_db_close();
	if (!$res || mysql_errno()) {
		return mysql_error();
	}
	return $res;

}

function fetch_saved_reports($type=false)
{
	if (isset($_SERVER['REMOTE_USER'])) {
		$user = $_SERVER['REMOTE_USER'];
	} else {
		return false;
	}

	$xtra_sql = "";
	$type = (int)$type;
	if ($type) {
		$xtra_sql = " AND ars.report_type_id=".$type;
	}
	$sql = "
		SELECT
			ars.id,
			ars.recipients,
			ars.report_type_id,
			ars.report_id,
			ars.period_id,
			ars.filename,
			ars.description,
			art.name AS report_type,
			arp.periodname
		FROM
			auto_reports_scheduled ars,
			auto_report_types art,
			auto_reports_periods arp
		WHERE
			ars.user='".$user."' AND
			arp.id=ars.period_id AND
			art.id=ars.report_type_id ".$xtra_sql."
		ORDER BY
			art.name";
	gui_db_connect();
	$res = sql_exec_query($sql);
	gui_db_close();
	if (!$res || mysql_errno()) {
		return mysql_error();
	}
	return $res;
}

function fetch_reports($type=false)
{
	// Instantiate the xajaxResponse object
	$objResponse = new xajaxResponse();
	$objResponse->call("show_progress", "progress", 'Please wait');

	// fetch info on selected type
	$res = fetch_saved_reports($type);

	if (!$res) {
		$objResponse->assign("output","innerHTML", "FAILED fetching reports");
		return $objResponse;
	}

	while ($row = sql_fetch_array($res)) {
		$return[] = $row;
	}

	$objResponse->call('show_saved_reports', convert_json($return));
	$objResponse->call('hide_progress');
	return $objResponse;

}

/**
 * Update specific field for certain scheduled report
 * Called from save_item.php by ajax
 *
 * @param 	in $id
 * @param 	str $field
 * @param 	mixed $value
 * @return 	bool
 */
function update_report_field($id=false, $field=false, $value=false)
{
	gui_db_connect();
	$id = (int)$id;
	$field = mysql_real_escape_string(trim($field));
	$value = mysql_real_escape_string(trim($value));
	$sql = "UPDATE auto_reports_scheduled SET `".$field."`= '".$value."' WHERE id=".$id;
	$res = sql_exec_query($sql);
	gui_db_close();
	if (!$res || mysql_errno()) {
		return mysql_error();
	}
	return true;
}

function get_typeof_report($id=false)
{
	$sql = "SELECT report_type_id FROM auto_reports_scheduled WHERE id=".$id;
	gui_db_connect();
	$res = sql_exec_query($sql);
	gui_db_close();
	if (!$res || mysql_errno()) {
		return mysql_error();
	}
	return mysql_num_rows($res)!=0 ? mysql_result($res, 0, 'report_type_id') : false;
}

/**
 * Helper function to wrap JSON conversion
 *
 * @param mixed $data
 * @param str $func [encode/decode]
 * @return str json_val
 */
function convert_json($data=false, $func='encode')
{
	if (empty($data)) return false;
	if (!function_exists('json_encode')) {
		require_once('/var/www/html/op5common/lib/json_class.php');
		$json = new JSON();
		$json_val = $json->$func($data);
	} else {
		$function = 'json_'.$func;
		$json_val = $function($data);
	}
	return $json_val;
}

function get_element_parts($str=false)
{
	if (empty($str)) return false;
	if (!strstr($str, '-')) return false;
	// check for report_name since it has '.' as element id
	if (strstr($str, '.')) {
		$dotparts = explode('.', $str);
		if (is_array($dotparts)) {
			$str = '';
			for ($i=1;$i<sizeof($dotparts);$i++) {
				$str .= $dotparts[$i];
			}
		}
	}
	$parts = explode('-', $str);
		if (is_array($parts)) {
			return $parts;
		}
	return false;
}

function convert_special_chars($str=false) {
	$str = trim($str);
	if (empty($str)) return false;
	$return_str = '';
	$str = trim($str);
	$str = str_replace(' ', '_', $str);
	$str = str_replace('"', '', $str);
	$str = str_replace('/', '_', $str);
	$str = utf8_decode($str);
	for ($i=0;$i<strlen($str);$i++) {
		if (ord($str[$i]) > 245) {
			$str[$i] = 'o';
		} elseif (ord($str[$i])>122) {
			$str[$i] = 'a';
		} else {
			$str[$i] = $str[$i];
		}
		$return_str .= $str[$i];
	}
	return $return_str;
}

function check_filename($str=false)
{
	$str = trim($str);
	$str = str_replace(',', '_', $str);
	if (empty($str)) return false;
	$extension = 'pdf';
	if (strstr($str, '.')) {
		$parts = explode('.', $str);
		if (is_array($parts)) {
			$str = '';
			for ($i=0;$i<(sizeof($parts)-1);$i++) {
				$str .= $parts[$i];
			}
			$str .= '.'.$extension;
		}
	} else {
		$str .= '.'.$extension;
	}
	return $str;
}

function fetch_field_value($type=false, $id=false, $elem_id=false)
{
	$id = (int)$id;
	$type = trim($type);
	$elem_id = trim($elem_id);
	if (empty($type) || empty($id) || empty($elem_id)) return false;
	$objResponse = new xajaxResponse();
	$sql = "SELECT * FROM auto_reports_scheduled WHERE id=".$id;
	gui_db_connect();
	$res = sql_exec_query($sql);
	gui_db_close();
	if (!$res || mysql_errno()) {
		return mysql_error();
	}
	$objResponse->call("show_progress", "progress", 'Please wait...');
	$row = sql_fetch_array($res);
	$objResponse->assign($elem_id,"innerHTML", $row[$type]);
	$objResponse->call('hide_progress');
	return $objResponse;
}
?>
