<?php

function changes_since_last_import () {
	global $F;
	global $HOME;
	$ift = 0;
	$rft = 0;

	clearstatcache ();
	foreach ($F as $k => $fd) {
		if (!file_exists ($fd)) return (true);
		$i = filemtime ($fd);
		if (!$ift || $ift > $i) $ift = $i;
	}

	$cfg_files = get_config_files ();
	foreach ($cfg_files as $cfg_file) {
		$i = filemtime ($HOME . "/" . $cfg_file);
		if ($i > $ift) return (true);
	}

	return (false);
}

/* swallows objects in non-static style. It's slower and more tedious,
 * but it's a onetime penalty, so compatibility is more important.
 * Now also keeps the top-level comments */
function import_objects ($fname) {
	global $comment_dir;
	global $HOME;
	global $Preserve_Vars;

	$ret_array = array();
	$inobj = false;
	$objs = 0;
	$in_topcomment = true;
	$fname = $HOME . "/" . $fname;

	if (!($buf = read_file ($fname))) return false;
	$lines = explode ("\n", $buf);

	foreach ($lines as $line) {
		/* Skip comments and 'out of object' data, unless we're
		 * still working on the topcomments (syntax explanation) */
		$line = trim ($line);
		if (!strlen($line) || $line[0] === '#' || !strlen($line)) continue;

		/* add object type */
		if (!strncmp ($line, 'define', strlen ('define'))) {
			$str = split ("[\t ]+", $line, 2);
			$obj_type = preg_replace ("/\s*{\s*/", '', $str[1]);
			if(isset($Preserve_Vars[$obj_type]))
			  $ret_array[$objs]['otype'] = $obj_type;
		}
		if(!isset($Preserve_Vars[$obj_type])) continue;

		if (strchr ($line, '{')) { $inobj = true; continue; }
		if (strchr ($line, '}')) { $inobj = false; $objs++; continue; }
		if ($inobj === false) continue;

		/* separate token and insert */
		$str = split ("[\t ]+", $line, 2);

		/* only get the interesting variables */
		if(my_array_search($str[0], $Preserve_Vars[$obj_type]))
		  $ret_array[$objs][$str[0]] = $str[1];
	}

	return $ret_array;
}

/* Get configuration files from main config file and return them in an array.
 * No object config files are read here. */
function get_config_files () {
	global $Config;
	global $HOME;

	$cfg_files = array ();

	/* read, mangle and put in array */
	$buf = read_file ($Config['main']);
	$lines = explode ("\n", $buf);
	foreach ($lines as $line) {
		/* skip junk, split string and evaluate */
		if ($line==='' || $line[0] == '#' || !strlen ($line)) continue;
		$str = split ("[\t ]*=[\t ]*", trim ($line), 2);
		if ($str[0] == "cfg_file") {
			$f = str_replace ($HOME, '', $str[1]);
			$cfg_files[] = $f;
		}
		/* found a dir, so get files from it (recursive?) */
		else if ($str[0] == "cfg_dir") {
			$dir_files = glob ($str[1] . "/*.cfg");
			foreach ($dir_files as $df) {
				$f = str_replace ($HOME, '', $df);
				$cfg_files[] = $f;
			}
		}
	} /* foreach ($lines as $line) */

	return ($cfg_files);
}

/* import objects from config files (uses get_config_files) */
function import_object_config_files () {
	global $Config;
	global $F;
	global $HOME;

	$cfg_files = get_config_files ();

	foreach ($cfg_files as $cfg_file)
	  $arr[$cfg_file] = import_objects($cfg_file);

	return ($arr);
}

/* caller / coordinator for the other functions */
function import_config () {
	global $F;
	global $IMPORTDIR;

	if(!is_dir($IMPORTDIR)) {
		if(!(mkdir($IMPORTDIR, 0750))) {
			echo "Failed to create missing directory $IMPORTDIR<br>\n";
			echo "Bailing out<br>\n";
			html_end();
		}
	}

	$arr = import_object_config_files();
	foreach ($arr as $cfg_file => $objs) {
		if (!is_array ($objs)) continue;
		foreach ($objs as $obj) {
			$tmp[$obj['otype']][] = $obj;
		}
	}
	$obj_types = array_keys ($tmp);
	foreach ($obj_types as $obj_type) {
		if(isset($F[$obj_type]))
		  glue_objects2 ($tmp[$obj_type], $obj_type);
	}

	/* free the 4 Mb or so... */
	$tmp = $arr = $objs = "";

	return (true);
}

?>
