%define daemon_user monitor
%define daemon_group apache

Summary: A tool that sends out reports from op5 monitor by weekly or monthly basis.
Name: monitor-autoreports
Version:  2.1.6
Release: op5.1
Group: op5/Monitor
License: op5 Immaterial Rights License
Vendor: op5 AB
Prefix: /opt/monitor/op5/auto-reports
Requires: php-mysql,httpd,monitor,monitor-gui >= 3.3.4
Requires: op5common >= 1.1.4
Requires: monitor-reports-gui >= 1.2.1
Source: %name-%version.tar.gz
BuildArch: noarch
BuildRoot: /override/%name-%version

%description
op5 Monitor Auto-reports makes it possible to customize reports and send them
to a list of arbitrary email addresses on a weekly or monthly basis.  You
define your reports using a simple web interface.  Reports are sent out
using a cron job.

%prep
%setup -q

%install
rm -rf %buildroot

mkdir -p -m 750 %buildroot%prefix
mkdir -p -m 770 %buildroot%prefix/{imported,schedules}
mkdir -p %buildroot%prefix/html2ps/{cache,out,tmp}

chmod 770 %buildroot%prefix/html2ps/{cache,out,tmp}
chmod 664 send-report.php

mv * %buildroot%prefix

%post
install -m 644 %prefix/auto-reports.crontab /etc/cron.d/auto-reports

# Verify that mysql-server is installed and running before executing sql scripts
$(mysql -Be "quit" 2>/dev/null)
if [ $? -eq 0 ]; then
# Create table if not exist
mysql monitor_gui < %prefix/auto_reports.sql
# Lazy... Alter table to support 255 chars in contact field.
mysql monitor_gui -e "ALTER TABLE auto_reports MODIFY contacts varchar(255)"
#mysql monitor_gui -e "ALTER TABLE auto_reports MODIFY id int(11) NOT NULL"
#mysql monitor_gui -e "ALTER TABLE auto_reports DROP PRIMARY KEY" >& /dev/null
#mysql monitor_gui -e "ALTER TABLE auto_reports ADD UNIQUE KEY (id, host, service, hostgrp, servicegrp)"
mysql monitor_gui -Be 'DESCRIBE auto_reports' | grep ^scheduleddowntimeasuptime &> /dev/null || \
			mysql monitor_gui < %prefix/upgrade_v1.4.4-to-1.6.0.sql
mysql monitor_gui -e "ALTER TABLE auto_reports MODIFY scheduleddowntimeasuptime  varchar(4) DEFAULT 'no'"
mysql monitor_gui -e "UPDATE auto_reports SET scheduleddowntimeasuptime='yes' WHERE scheduleddowntimeasuptime='1'"
mysql monitor_gui -e "UPDATE auto_reports SET scheduleddowntimeasuptime='no' WHERE scheduleddowntimeasuptime='0'"


if ! $(mysql monitor_gui -Be "SHOW TABLES" | grep "auto_reports_db_version" > /dev/null); then
	mysql monitor_gui < %prefix/autoreports.sql

	# convert old scheduled reports to v2.0 format:
	# 1  SELECT * FROM auto_reports
	# 2a INSERT INTO avail_config
	# 2b INSERT INTO avail_config_objects
	# 2c INSERT INTO auto_reports_scheduled
	/usr/bin/php %prefix/convert_to_2.0.php
	echo "Old scheduled reports has been imported and converted to new format!"
fi


# Check version
query='SELECT version FROM auto_reports_db_version'
ver=$(mysql monitor_gui -Be "$query" 2>/dev/null | sed -n \$p)
case "$ver" in
    2.0.0)
    	# Add description field to auto_reports_scheduled
        mysql monitor_gui < %prefix/upgrade_2.0.1-to-2.0.2.sql
		query="UPDATE auto_reports_db_version SET version='2.1.1'"
		mysql monitor_gui -Be "$query"  > /dev/null
        ;;
	2.0.2)
		# Add support for Daily reports
		sql='SELECT * FROM auto_reports_periods';
		if ! $(mysql monitor_gui -Be "$sql" | grep "Daily" > /dev/null); then
			mysql monitor_gui < %prefix/upgrade_2.1.0-to-2.1.1.sql
			query="UPDATE auto_reports_db_version SET version='2.1.1'"
			mysql monitor_gui -Be "$query"  > /dev/null
		fi
		;;
esac
fi

# Import scheduled reports from old format
unset print
rm -rf "%prefix/schedules"
# Import completed


if [ ${print} ]; then
	echo "Old scheduled reports has been imported!"
	echo "NOTE! Reports will be generated as the webuser 'monitor'. If that user"
	echo "do not exist or has insufficient privileges report generation will fail."
fi


%postun
# Only remove crontab-entry when un-installing, not on upgrade
if [ $1 -eq 0 ]; then
	rm -rf /etc/cron.d/auto-reports
fi
if  [ -f /var/spool/cron/monitor ]; then
	crontab -l -u monitor | grep -v "/opt/monitor/op5/auto-reports/send-report.php" > /tmp/cron-tmp
	crontab -u monitor /tmp/cron-tmp
	rm -rf /tmp/cron-tmp
fi



%files
%defattr(-,%daemon_user,%daemon_group)
%prefix


%clean
rm -rf %buildroot


%changelog
* Thu May 03 2007 Peter Ostlin <peter@op5.se>
- changing to group apache to comply with centos
- Require httpd not apache to comply with centos
* Fri Oct 15 2004 Andreas Ericsson <ae@op5.se> 1.1-op5.1
- First working version.
